@@ -2,7 +2,7 @@
 
 package Exception::Base;
 use 5.006;
-our $VERSION = 0.14;
+our $VERSION = 0.15;
 
 =head1 NAME
 
@@ -12,7 +12,7 @@ Exception::Base - Lightweight exceptions
 
   # Use module and create needed exceptions
   use Exception::Base (
-    ':all',                            # import try/catch functions
+    ':all',                            # import try/catch/throw
     'Exception::Runtime',              # create new module
     'Exception::System',               # load existing module
     'Exception::IO',          => {
@@ -24,9 +24,9 @@ Exception::Base - Lightweight exceptions
 
   # try / catch
   try eval {
-    do_something() or throw Exception::FileNotFound
-                                message=>'Something wrong',
-                                tag=>'something';
+    do_something() or throw 'Exception::FileNotFound' =>
+                            message=>'Something wrong',
+                            tag=>'something';
   };
   # Catch the Exception::Base and derived, rethrow immediately others
   if (catch my $e) {
@@ -40,7 +40,7 @@ Exception::Base - Lightweight exceptions
   }
 
   # the exception can be thrown later
-  $e = new Exception::Base;
+  $e = Exception::Base->new;
   # (...)
   $e->throw;
 
@@ -57,12 +57,12 @@ Exception::Base - Lightweight exceptions
 
   # don't use syntactic sugar
   use Exception::Base;          # does not import ':all' tag
-  try Exception::Base eval {
-    throw Exception::IO;
-  };
-  catch Exception::Base my $e;  # catch Exception::Base and derived
+  Exception::Base->try(eval {
+    Exception::IO->throw;
+  });
+  Exception::Base->catch(my $e);  # catch Exception::Base and derived
   # or
-  catch Exception::IO my $e;    # catch IO errors and rethrow others
+  Exception::IO->catch(my $e);    # catch IO errors and rethrow others
 
   # run Perl with changed verbosity
   sh$ perl -MException::Base=verbosity,4 script.pl
@@ -102,10 +102,6 @@ implements error stack, the try/catch blocks can be nested
 
 =item *
 
-shows full backtrace stack on die by default
-
-=item *
-
 the default behaviour of exception class can be changed globally or just for
 the thrown exception
 
@@ -146,7 +142,7 @@ BEGIN { *Symbol::fetch_glob = sub ($) { no strict 'refs'; \*{$_[0]} } unless def
 
 # Syntactic sugar
 use Exporter ();
-our @EXPORT_OK = qw< try catch >;
+our @EXPORT_OK = qw< try catch throw >;
 our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 
 
@@ -202,7 +198,7 @@ sub import {
 
     while (defined $_[0]) {
         my $name = shift @_;
-        if ($name =~ /^(try|catch|:all)$/) {
+        if ($name =~ /^(try|catch|throw|:all)$/) {
             push @export, $name;
         }
         elsif ($name =~ /^([+-]?)([a-z0-9_]+)$/) {
@@ -318,7 +314,7 @@ sub unimport {
         if ($name eq ':all') {
             unshift @export, @EXPORT_OK;
         }
-        elsif ($name eq 'try' or $name eq 'catch') {
+        elsif ($name eq 'try' or $name eq 'catch' or $name eq 'throw') {
             if (defined *{Symbol::fetch_glob($callpkg . '::' . $name)}{CODE}) {
                 # Store and restore other typeglobs than CODE
                 my %glob;
@@ -387,7 +383,7 @@ sub new {
 
 
 # Create the exception and throw it or rethrow existing
-sub throw {
+sub throw (;$@) {
     my $self = shift;
 
     my $old;
@@ -565,7 +561,7 @@ sub with {
 
 
 # Push the exception on error stack. Stolen from Exception::Class::TryCatch
-sub try ($) {
+sub try ($;$) {
     # Can be used also as function
     my $self = shift if defined $_[0] and do { local $@; local $SIG{__DIE__}; eval { $_[0]->isa(__PACKAGE__) } };
 
@@ -577,7 +573,7 @@ sub try ($) {
 
 
 # Pop the exception on error stack. Stolen from Exception::Class::TryCatch
-sub catch {
+sub catch (;$$) {
     # Can be used also as function
     my $self = shift if defined $_[0] and do { local $@; local $SIG{__DIE__}; eval { $_[0]->isa(__PACKAGE__) } };
 
@@ -973,12 +969,12 @@ __END__
 
 =over
 
-=item use Exception::Base qw< catch try >;
+=item use Exception::Base qw< catch try throw >;
 
-Exports the B<catch> and B<try> functions to the caller namespace.
+Exports the B<catch>, B<try> and B<throw> functions to the caller namespace.
 
-  use Exception::Base qw< catch try >;
-  try eval { throw Exception::Base; };
+  use Exception::Base qw< catch try throw >;
+  try eval { throw 'Exception::Base'; };
   if (catch my $e) { warn "$e"; }
 
 =item use Exception::Base ':all';
@@ -1029,7 +1025,7 @@ creates the exception class automatically at compile time.  The newly created
 class will be based on L<Exception::Base> class.
 
   use Exception::Base qw< Exception::Custom Exception::SomethingWrong >;
-  throw Exception::Custom;
+  Exception::Custom->throw;
 
 =item use Exception::Base 'I<Exception>' => { isa => I<BaseException>, version => I<version>, ... };
 
@@ -1071,13 +1067,13 @@ The class will have the default property for the given field.
     'Exception::FileNotFound' => { isa => 'Exception::IO' },
     'Exception::My' => { version => 0.2 },
     'Exception::WithDefault' => { message => 'Default message' };
-  try eval { throw Exception::FileNotFound; };
+  try eval { Exception::FileNotFound->throw; };
   if (catch my $e) {
     if ($e->isa('Exception::IO')) { warn "can be also FileNotFound"; }
     if ($e->isa('Exception::My')) { print $e->VERSION; }
   }
 
-=item no Exception::Base qw< catch try >;
+=item no Exception::Base qw< catch try throw >;
 
 =item no Exception::Base ':all';
 
@@ -1086,9 +1082,9 @@ The class will have the default property for the given field.
 Unexports the B<catch> and B<try> functions from the caller namespace.
 
   use Exception::Base ':all', 'Exception::FileNotFound';
-  try eval { throw Exception::FileNotFound; };  # ok
+  try eval { Exception::FileNotFound->throw; };  # ok
   no Exception::Base;
-  try eval { throw Exception::FileNotFound; };  # syntax error
+  try eval { Exception::FileNotFound->throw; };  # syntax error
 
 =back
 
@@ -1137,7 +1133,7 @@ fields.
   package main;
   use Exception::Base ':all';
   try eval {
-    throw Exception::My readonly=>1, readwrite=>2;
+    throw 'Exception::My' => readonly=>1, readwrite=>2;
   };
   if (catch my $e) {
     print $e->{readwrite};                # = 2
@@ -1159,7 +1155,7 @@ available as accessors methods.
 Contains the message of the exception.  It is the part of the string
 representing the exception object.
 
-  eval { throw Exception message=>"Message", tag=>"TAG"; };
+  eval { Exception::Base->throw(message=>"Message", tag=>"TAG"); };
   print $@->{message} if $@;
 
 =item properties (ro)
@@ -1167,7 +1163,7 @@ representing the exception object.
 Contains the additional properies of the exception.  They can be later used
 with "with" method.
 
-  eval { throw Exception message=>"Message", tag=>"TAG"; };
+  eval { Exception::Base->throw(message=>"Message", tag=>"TAG"); };
   print $@->{properties}->{tag} if $@;
 
 =item verbosity (rw, default: 2)
@@ -1255,7 +1251,7 @@ called.
 
   package My::Package;
   use Exception::Base;
-  throw Exception::Base ignore_class => "My::Base";
+  Exception::Base->throw(ignore_class => "My::Base");
 
 This setting can be changed with import interface.
 
@@ -1282,7 +1278,7 @@ line-feed (\n) removed from its message.
 
   try eval {
     eval { $a = $b = 0; $c = $a / $b };
-    throw Exception::Base;
+    Exception::Base->throw;
   };
   catch my $e;
   print $e->eval_error;
@@ -1292,7 +1288,7 @@ line-feed (\n) removed from its message.
 Contains the timestamp of the thrown exception.  Collected if the verbosity
 on throwing exception was greater than 1.
 
-  eval { throw Exception message=>"Message"; };
+  eval { Exception::Base->throw(message=>"Message"); };
   print scalar localtime $@->{time};
 
 =item pid (ro)
@@ -1300,7 +1296,7 @@ on throwing exception was greater than 1.
 Contains the PID of the Perl process at time of thrown exception.  Collected
 if the verbosity on throwing exception was greater than 1.
 
-  eval { throw Exception message=>"Message"; };
+  eval { Exception::Base->throw(message=>"Message"); };
   kill 10, $@->{pid};
 
 =item tid (ro)
@@ -1329,7 +1325,7 @@ and are the arguments of called function.  Collected if the verbosity on
 throwing exception was greater than 1.  Contains only the first element of
 caller stack if the verbosity was lower than 3.
 
-  eval { throw Exception message=>"Message"; };
+  eval { Exception::Base->throw(message=>"Message"); };
   ($package, $filename, $line, $subroutine, $hasargs, $wantarray,
   $evaltext, $is_require, @args) = $@->{caller_stack}->[0];
 
@@ -1338,7 +1334,7 @@ caller stack if the verbosity was lower than 3.
 Contains the maximal length of argument for functions in backtrace output.
 Zero means no limit for length.
 
-  sub a { throw Exception max_arg_len=>5 }
+  sub a { Exception::Base->throw(max_arg_len=>5) }
   a("123456789");
 
 =item max_arg_nums (rw, default: 8)
@@ -1346,7 +1342,7 @@ Zero means no limit for length.
 Contains the maximal number of arguments for functions in backtrace output.
 Zero means no limit for arguments.
 
-  sub a { throw Exception max_arg_nums=>1 }
+  sub a { Exception::Base->throw(max_arg_nums=>1) }
   a(1,2,3);
 
 =item max_eval_len (rw, default: 0)
@@ -1354,14 +1350,14 @@ Zero means no limit for arguments.
 Contains the maximal length of eval strings in backtrace output.  Zero means
 no limit for length.
 
-  eval "throw Exception max_eval_len=>10";
+  eval "Exception->throw(max_eval_len=>10)";
   print "$@";
 
 =item defaults (rw)
 
 Meta-field contains the list of default values.
 
-  my $e = new Exception;
+  my $e = Exception::Base->new;
   print defined $e->{verbosity}
     ? $e->{verbosity}
     : $e->{defaults}->{verbosity};
@@ -1380,8 +1376,10 @@ fields like B<time>, B<pid>, B<uid>, B<gid>, B<euid>, B<egid> are not filled.
 If the key of the argument is read-write field, this field will be filled.
 Otherwise, the properties field will be used.
 
-  $e = new Exception message=>"Houston, we have a problem",
-                     tag => "BIG";
+  $e = Exception::Base->new(
+           message=>"Houston, we have a problem",
+           tag => "BIG"
+       );
   print $e->{message};
   print $e->{properties}->{tag};
 
@@ -1394,7 +1392,12 @@ values for the class are also stored in internal cache.
 Creates the exception object and immediately throws it with die() function.
 
   open FILE, $file
-    or throw Exception message=>"Can not open file: $file";
+    or Exception::Base->throw(message=>"Can not open file: $file");
+
+This method is also exported as a function.
+
+  open FILE, $file
+    or throw 'Exception::Base' => message=>"Can not open file: $file";
 
 =back
 
@@ -1408,11 +1411,11 @@ Immediately throws exception object.  It can be used for rethrowing existing
 exception object.  Additional arguments will override the fields in existing
 exception object.
 
-  $e = new Exception::Base;
+  $e = Exception::Base->new;
   # (...)
   $e->throw(message=>"thrown exception with overriden message");
 
-  eval { throw Exception::Base message=>"Problem", fatal=>1 };
+  eval { Exception::Base->throw(message=>"Problem", fatal=>1) };
   $@->throw if $@->properties->{fatal};
 
 =item throw($I<exception>, [%I<args>])
@@ -1420,9 +1423,9 @@ exception object.
 Immediately rethrows an existing exception object as an other exception
 class.
 
-  eval { open $f, "w", "/etc/passwd" or throw Exception::System };
+  eval { open $f, "w", "/etc/passwd" or Exception::System->throw };
   # convert Exception::System into Exception::Base
-  throw Exception::Base $@;
+  Exception::Base->throw($@);
 
 =item stringify([$I<verbosity>[, $I<message>]])
 
@@ -1430,7 +1433,7 @@ Returns the string representation of exception object.  It is called
 automatically if the exception object is used in scalar context.  The method
 can be used explicity and then the verbosity level can be used.
 
-  eval { throw Exception; };
+  eval { Exception::Base->throw; };
   $@->{verbosity} = 1;
   print "$@";
   print $@->stringify(4) if $VERY_VERBOSE;
@@ -1467,23 +1470,29 @@ The B<try> method or function can be used with eval block as argument.  Then
 the eval's error is pushed into error stack and can be used with B<catch>
 later.
 
-  try Exception::Base eval { throw Exception; };
+  Exception::Base->try(eval { Exception::Base->throw; });
   eval { die "another error messing with \$@ variable"; };
-  catch Exception::Base my $e;
+  Exception::Base->catch(my $e);
 
 The B<try> returns the value of the argument in scalar context.  If the
 argument is array reference, the B<try> returns the value of the argument in
 array context.
 
-  $v = try Exception::Base eval { 2 + 2; }; # $v == 4
-  @v = try Exception::Base [ eval { (1,2,3); }; ]; # @v = (1,2,3)
+  $v = Exception::Base->try( eval { 2 + 2; } ); # $v == 4
+  @v = Exception::Base->try( [ eval { (1,2,3); }; ] ); # @v = (1,2,3)
 
 The B<try> can be used as method or function.
 
-  try Exception::Base eval { throw Exception::Base "method"; };
-  Exception::Base::try eval { throw Exception::Base "function"; };
+  try 'Exception::Base' => eval {
+    Exception::Base->throw(message=>"method"); 
+  };
+  Exception::Base::try eval {
+    Exception::Base->throw(message=>"function");
+  };
   Exception::Base->import('try');
-  try eval { throw Exception::Base "exported function"; };
+  try eval {
+    Exception::Base->throw(message=>"exported function");
+  };
 
 =item I<CLASS>->catch([$I<variable>])
 
@@ -1491,16 +1500,16 @@ The exception is popped from error stack written into the method argument.  If
 the exception is not based on the I<CLASS>, the exception is thrown
 immediately.
 
-  try eval { throw Exception; };
+  try eval { Exception::Base->throw; };
   catch Exception::Base my $e;
   print $e->stringify(1);
 
 If the error stack is empty, the B<catch> method returns undefined value.  It
 can be used in loop to clean up all unhandled exceptions.
 
-  try eval { -f 'file1' or throw Exception::FileNotFound };
-  try eval { -f 'file2' or throw Exception::FileNotFound };
-  try eval { -f 'file3' or throw Exception::FileNotFound };
+  try eval { -f 'file1' or Exception::FileNotFound->throw };
+  try eval { -f 'file2' or Exception::FileNotFound->throw };
+  try eval { -f 'file3' or Exception::FileNotFound->throw };
   while (catch my $e) {
       warn "$e" if not $e->isa('Exception::FileNotFound');
   }
@@ -1510,28 +1519,27 @@ exception object is created with message from B<$@> variable with removed
 C<" at file line 123."> string and the last end of line (LF).
 
   try eval { die "Died\n"; };
-  catch Exception::Base my $e;
+  catch 'Exception::Base', my $e;
   print $e->stringify;
 
 The method returns B<1>, if the exception object is caught, and returns B<0>
 otherwise.
 
-  try eval { throw Exception; };
-  if (catch Exception::Base my $e) {
+  try eval { throw 'Exception::Base'; };
+  if (Exception::Base->catch(my $e)) {
     warn "Exception caught: " . ref $e;
-  }
 
 If the method argument is missing, the method returns the exception object.
 
-  try eval { throw Exception; };
-  my $e = catch Exception::Base;
+  try eval { Exception::Base->throw; };
+  my $e = Exception::Base->catch;
 
 The B<catch> can be used as method or function.  If it is used as function,
 then the I<CLASS> is Exception::Base by default.
 
-  try eval { throw Exception::Base "method"; };
+  try eval { throw 'Exception::Base' => message=>"method"; };
   Exception::Base->import('catch');
-  catch my $e;  # the same as catch Exception::Base my $e;
+  catch my $e;  # the same as Exception::Base->catch(my $e);
   print $e->stringify;
 
 =item I<CLASS>->catch([$I<variable>,] \@I<ExceptionClasses>)
@@ -1541,7 +1549,7 @@ stack is empty.  If the exception is not based on the I<CLASS> and is not
 based on one of the class from argument, the exception is thrown immediately.
 
   try eval { throw Exception::IO; }
-  catch Exception::Base my $e, ['Exception::IO'];
+  catch 'Exception::Base', my $e, ['Exception::IO'];
   print "Only IO exception was caught: " . $e->stringify(1);
 
 =item package
